// NEOPIXEL BEST PRACTICES for most reliable operation:
// - Add 1000 uF CAPACITOR between NeoPixel strip's + and - connections.
// - MINIMIZE WIRING LENGTH between microcontroller board and first pixel.
// - NeoPixel strip's DATA-IN should pass through a 300-500 OHM RESISTOR.
// - AVOID connecting NeoPixels on a LIVE CIRCUIT. If you must, ALWAYS
//   connect GROUND (-) first, then +, then data.
// - When using a 3.3V microcontroller with a 5V-powered NeoPixel strip,
//   a LOGIC-LEVEL CONVERTER on the data line is STRONGLY RECOMMENDED.
// (Skipping these may work OK on your workbench but can fail in the field)
#include <Wire.h>
#include <SPI.h>
#include <Adafruit_LIS3DH.h>
#include <FastLED.h>
#include <Adafruit_SleepyDog.h>

#ifdef __AVR__
 #include <avr/power.h> // Required for 16 MHz Adafruit Trinket
#endif

// Which pin on the Arduino is connected to the NeoPixels?
// On a Trinket or Gemma we suggest changing this to 1:
#define PROPMAKER_PIN    10
#define LED_PIN    5
#define RED_LED_PIN    13
// VBATPIN is used to measure the battery voltage
#define VBATPIN A9

// How many NeoPixels are attached to the Arduino?
#define LED_COUNT 240
#define BRIGHTNESS  100
//in what order are the color values defined for each LED
//if you colors don't look right, try changing this
#define COLOR_ORDER GRB 

int Brightness = 0;

// This is an array of leds.  One item for each led in your strip.
CRGB leds[LED_COUNT];
//keep track of which Effect to play next
byte selectedEffect=0;

// Adjust this number for the sensitivity of the 'click' force
// this strongly depend on the range! for 16G, try 5-10
// for 8G, try 10-20. for 4G try 20-40. for 2G try 40-80
#define CLICKTHRESHHOLD 30
//address of the LIS3DH
int LIS3DH_ADDR = 0x18;

// inst. i2c accelerometer
Adafruit_LIS3DH lis = Adafruit_LIS3DH();

int interruptPin = 6;                 // Setup pin D6 to be the interrupt pin 

//variables used to keep track of time
unsigned long currentClickMillis;
unsigned long clickMillis;
unsigned long lastClickMillis;
unsigned long listExitCheckMillis=0;
unsigned long startPlayingMillis=0;
const unsigned long period = 300;
unsigned long SleepPeriod = 60 * 1000; //60 seconds

int staffState = 0;

/* Initialize accelerometer in the host processor. ----------------------------------*/
//It only needs to be executed one time after power up in initialization routine.
void init_ACC(void)
{
       // configurations for control registers
    lis.writeRegister8(LIS3DH_REG_CTRL1, 0x57);                       // Write 0x57 - turn on x, y, z sensor axis with ODR of 100 Hz
    lis.writeRegister8(LIS3DH_REG_CTRL2, 0x00);                       // High-pass filter (HPF) enabled (0x09) or disabled (0x00)
    lis.writeRegister8(LIS3DH_REG_CTRL3, 0x40);                       // ACC AOI1 interrupt signal is routed to INT1 pin. (I1_IA1)
    lis.writeRegister8(LIS3DH_REG_CTRL4, 0x30);                       // Full Scale = +/-2G (2G=0x00, 4G=0x10, 8G=0x20, 16G=0x30)
    lis.writeRegister8(LIS3DH_REG_CTRL5, 0x08);                       
    lis.writeRegister8(LIS3DH_REG_INT1THS, 0x01);                     // Threshold 250 mg (1 LSB equels xx mg. 2G = 16, 4G = 32, 8G = 262, 16G = 186)
    lis.writeRegister8(LIS3DH_REG_INT1DUR, 0x10);                     // Event(!) duration - can be set to avoid false detections (e.g. to prevent false freefall detections). Duration = 1LSBs * (1/10Hz) = 0.1s.
    lis.writeRegister8(LIS3DH_REG_INT1CFG, 0x2A);                     /// Enable XLIE, YLIE, ZLIE interrupt 
}


// setup() function -- runs once at startup --------------------------------
void setup() {
  long wait=0;
  // initialize digital pin 13 as an output.
  pinMode(LED_PIN, OUTPUT);
  pinMode(PROPMAKER_PIN, OUTPUT);

  digitalWrite(interruptPin, LOW); 
  pinMode(interruptPin, INPUT);
  attachInterrupt(digitalPinToInterrupt(interruptPin), D6_ISR, RISING );   // Attach Interrupt - NOT WORKING!!!

  digitalWrite(PROPMAKER_PIN, HIGH); //turn on the propMaker!!!!

  // These lines are specifically to support the Adafruit Trinket 5V 16 MHz.
  // Any other board, you can remove this part (but no harm leaving it):
#if defined(__AVR_ATtiny85__) && (F_CPU == 16000000)
  clock_prescale_set(clock_div_1);
#endif
  // END of Trinket-specific code.
  Brightness= BRIGHTNESS;
  FastLED.setBrightness(Brightness );
  FastLED.addLeds<WS2811, LED_PIN, COLOR_ORDER>(leds, LED_COUNT);

  int setup=1;
  int wait_count=0;
  ShowSetup(setup++);  
  wait = millis();

  Serial.begin(115200);
  
  while ((!Serial)&&(wait_count++<700)) {
     delay(5);
     //millis()-wait<10000)
     if(wait_count%100==0)
        ShowSetup(++setup);
     
   }
   Serial.println("LightStaff v1.3");
  ShowSetup(++setup);
  
  if (! lis.begin(LIS3DH_ADDR)) {   // change this to 0x19 for alternative i2c address
    Serial.println("Couldnt start");
    ShowSetup(-1); //show some sort of error by setting leds to something wierd
    while (1); //stop here cause nothing works without the LIS3DH
  }
  Serial.println("LIS3DH found!");

  //was commented out
  //init_ACC();
  
  ShowSetup(++setup);
  
  lis.setRange(LIS3DH_RANGE_16_G);   // 2, 4, 8 or 16 G!
  
  Serial.print("Range = "); Serial.print(2 << lis.getRange());  
  Serial.println("G");

  // 0 = turn off click detection & interrupt
  // 1 = single click only interrupt output
  // 2 = double click only interrupt output, detect single click
  // Adjust threshhold, higher numbers are less sensitive
  // setClick(uint8_t c, uint8_t clickthresh, uint8_t timelimit = 10, uint8_t timelatency = 20, uint8_t timewindow = 255);
  lis.setClick(1, CLICKTHRESHHOLD, 10, 20, 20);
  lis.writeRegister8(LIS3DH_REG_CTRL3, 0x40);// ACC AOI1 interrupt signal is routed to INT1 pin. (I1_IA1)
  ShowSetup(++setup);
  delay(100);

    ShowSetup(-2); //done with setup, clear all LEDs
  
}

//trying to figure out what the State Of Charge is with the attached battery
float batteryCheck(boolean SoC){   
  float measuredvbat = analogRead(VBATPIN);
  float voltage = 0;
  float s = 0;
  Serial.print("RawRead: " ); Serial.println(measuredvbat);
  voltage = measuredvbat;
  voltage *= 2;    // we divided by 2, so multiply back
  voltage *= 3.3;  // Multiply by 3.3V, our reference voltage
  voltage /= 1024; // convert to voltage
  Serial.print("VBat: " ); Serial.println(voltage);
  s = map(measuredvbat, 500, 649, 0, 100);
  Serial.print("SoC: " ); Serial.println(s);
  if(SoC){
    return s;
  } else
    return measuredvbat;
}

// loop() function -- runs repeatedly as long as board is on ---------------
void loop() {
  bool isClick = checkClick();  //has the staff been clicked?
  int b = 0;
  digitalWrite(13, HIGH);   // turn the LED on (HIGH is the voltage level)
  switch(staffState){
    case 0: //no click yet but check to see if one occured
             if (isClick) {  
                //1st click seen for this loop
                lightTheTop();
                lastClickMillis = millis();
                staffState++;

             }
             else {
                //Serial.print("Nothing for ");
                //Serial.print(millis() - lastClickMillis, DEC);
                //Serial.println(" milliseconds now.");

                //IF THE LIS3DH INTERRUPT CODE STARTS WORKING THIS WILL GO AWAY
                //no click and we haven't seen one for a minute
                //lets add in some sleep code to save the battery
                if( millis() - lastClickMillis > 120000){
                    ShowSetup(-2); //turn off all LEDS
                    digitalWrite(13, LOW);   // turn the LED on (HIGH is the voltage level)
                    Serial.println("No activity, sleeping for 5 seconds.");
                    int sleepMS = Watchdog.sleep(8000);  // Sleep for up to 10 seconds.
                    Watchdog.sleep(8000);  // Sleep for up to 10 seconds.
                    Watchdog.sleep(8000);  // Sleep for up to 10 seconds.
                    Watchdog.sleep(6000);  // Sleep for up to 10 seconds.
                    USBDevice.attach();
                    Serial.print("I'm awake now! I slept for ");
                    Serial.print(sleepMS, DEC);
                    Serial.println(" milliseconds.");
                    Serial.println();
                    lastClickMillis = millis() - 110000; //once it wakes from the 8-10 sec sleep, we have about 3-4 to get it out of sleep mode
               } 
               else if( millis() - lastClickMillis > 119000){
                  ShowSetup(1);
                  Serial.println("Sleeping in 1 second.");
               }
               else if( millis() - lastClickMillis > 118000){
                  ShowSetup(2);
                  Serial.println("Sleeping in 2 seconds.");
               }
               else if( millis() - lastClickMillis > 117000){
                  ShowSetup(3);
                  Serial.println("Sleeping in 3 seconds.");
               }
               else if( millis() - lastClickMillis > 116000){
                  ShowSetup(4);
                  Serial.println("Sleeping in 4 seconds.");
               }
               else if( millis() - lastClickMillis > 115000){
                  ShowSetup(5);
                  Serial.println("Sleeping in 5 seconds.");
               }
               else if( millis() - lastClickMillis > 114000){
                  ShowSetup(6);
                  Serial.println("Sleeping in 6 seconds.");
               }
               else if( millis() - lastClickMillis > 113000){
                  ShowSetup(7);
                  Serial.println("Sleeping in 7 seconds.");
               }
               else if( millis() - lastClickMillis > 112000){
                  ShowSetup(8);
                  Serial.println("Sleeping in 8 seconds.");
               }
               else if( millis() - lastClickMillis > 111000){
                  ShowSetup(9);
                  Serial.println("Sleeping in 9 seconds.");
               }
               else if( millis() - lastClickMillis > 110000){
                  ShowSetup(10);
                  Serial.println("Sleeping in 10 seconds.");
               }
               ////////////////// END OF THE CODE TO FAKE A SLEEP //////////////////////////
             }
            break;
    case 1: //one click, play the stepTaken OR play the Effect if we get another click quick enough
           if (isClick){ 
                  if( millis() - lastClickMillis < period){
                      startPlayingMillis = millis();
                      Serial.print(" double click");
                      Serial.println();
                      FastLED.setBrightness(Brightness );
                      playEffect();
                      FastLED.setBrightness(Brightness/2 );
                      staffState = 0; //reset the state back to zero
                  }
            } else {
                  if( millis() - lastClickMillis >= period){
                      Serial.print(" single click");
                      Serial.println();
                      FastLED.setBrightness(Brightness/2 );
                      stepTaken();
                      Serial.print(" after stepTaken");
                      Serial.println();
                      FastLED.setBrightness(5);
                      staffState = 0; //reset the state back to zero
                      b=batteryCheck(true);
                      if (b<=10) {
                                Brightness = 40;  //turn down the LEDs brightness so the battery will last longer
                                for(int x=0;x<10;x++){
                                  setPixel(x, 250, 0, 0);          //mirror the LEDs
                                  setPixel(LED_COUNT-x, 250, 0, 0);//mirror the LEDs
                                }
                                showStrip();
                      } else if(b<=40) {
                                Brightness = 60; //turn down the LEDs brightness just a bit to save the battery
                                //rgb(255,255,153)  yellow
                                for(int x=0;x<10;x++){
                                  setPixel(x, 0,0,255);           //mirror the LEDs
                                  setPixel(LED_COUNT-x, 0,0,255);//mirror the LEDs
                                }
                                showStrip();
                                                                
                      }
                      FastLED.setBrightness(Brightness/2 );
                  }
            }
            break;
    case 2: //two clicks
            break;
            
  }

  delay(50);
}

//show some sort of error by setting leds to something wierd
void ShowSetup(int i){

  setAll(0,0,0);
  if(i>=0){
    for(int x=0;x<i;x++){
      setPixel(x, 0, 0, 255);
      setPixel(LED_COUNT-x, 0, 0, 255);
    }
    
  } else if (i==-1) {
    setAll(0,0,100);
  } else {
    setAll(0,0,0);
  }
    showStrip();
}

//light up the top when a step is seen, this lets the user know the tap was felt
void lightTheTop(){
    setAll(0,0,0);
    showStrip();
    // Fade IN
    for(int k = 0; k < 256; k=k+10) { 
      for(int z=0;z<10;z++){
          setPixel((LED_COUNT/2)-z, k, k, k);
          setPixel((LED_COUNT/2)+z, k, k, k);
      }
      showStrip();
      delay(3);
    }

  //this code just lights up the to 10 LEDs 
//  for(int whiteLed = 0; whiteLed < 10; whiteLed = whiteLed + 2) {
//      // Turn our current led on to white, then show the leds
//      leds[(LED_COUNT/2)-whiteLed] = CRGB::White;
//      leds[(LED_COUNT/2)+whiteLed]  = CRGB::White;
//      // Show the leds (only one of which is set to white, from above)
//      FastLED.show();
//
//      // Wait a little bit
//      delay(22);
//
//      // Turn our current led back to black for the next loop around
//      leds[(LED_COUNT/2)-whiteLed]  = CRGB::Black;
//      leds[(LED_COUNT/2)+whiteLed]  = CRGB::Black;
//   }
}

//check to see if there has been a tap, this will go away if the interrupt code starts working
bool checkClick() {
  uint8_t click = lis.getClick();
  if (click & 0x10){ 
    Serial.print("Click detected (0x"); Serial.print(click, HEX); Serial.print("): ");
    Serial.println();
    clickMillis = millis();
    return true;
  }
  return false;
}

// check to see if another click has occured, BUT lets only do this 10 times a sec
bool exitPlaying() {
  unsigned long currentMills = millis();
  //only check for another click 10 times a second
  if(currentMills - listExitCheckMillis > 100) {
    //Serial.println("exitPlaying check");
    listExitCheckMillis = currentMills; //update last check time
    if(checkClick()) {
      if((currentMills - startPlayingMillis)>1000){
        Serial.println("Exit Playing");
        return true;
      }
        
      Serial.println("Too early, DONT Exit Playing");
    }
  }
  return false;
}

//for some reason I can't get this to work, if you do then a most of the 'checkClick()' can be changed and a proper sleep written
void D6_ISR() {
  Serial.println(" !!!!!!!!!!!!!!!!!!!! pin_ISR() has been called !!!!!!!!!!!!!!!!!!!!!!!!!!!!");
}

// Some functions of our own for creating animated effects -----------------
//
//any time a 'step' or single click happens we light up the top then do 'meteorRain' to quickly shoot
//a white light down the length of the staff
void stepTaken(){
  meteorRain(0xff,0xff,0xff,2, 500, true, 0);
  // Move a single white led 
//   for(int whiteLed = 0; whiteLed < LED_COUNT/2; whiteLed = whiteLed + 2) {
//      // Turn our current led on to white, then show the leds
//      leds[(LED_COUNT/2)-whiteLed] = CRGB::White;
//      leds[(LED_COUNT/2)+whiteLed]  = CRGB::White;
//      // Show the leds (only one of which is set to white, from above)
//      FastLED.show();
//
//      // Wait a little bit
//      delay(1);
//
//      // Turn our current led back to black for the next loop around
//      leds[(LED_COUNT/2)-whiteLed]  = CRGB::Black;
//      leds[(LED_COUNT/2)+whiteLed]  = CRGB::Black;
//   }
//   setAll(0,0,0);
}

//function that shows the effect on the staff depending on the selectedEffect
//lots of these are commented out or switched to others that I liked the best but you can set your own
void playEffect(){
  //selectedEffect=15;  //uncomment this to high jack the effect that will be played
  setAll(0,0,0);
  showStrip();
  switch(selectedEffect) {
    case 0  : 
//                // RGBLoop - no parameters
//                Serial.println("      RGBLoop");
//                RGBLoop();
//                Serial.println("      RGBLoop Exit");
//                break;
                Serial.println("      Fire");
                // Fire - Cooling rate, Sparking rate, speed delay
                Fire(80,120,2,4);
                Serial.println("     Fire Exit");
                break;
              

    case 1  : 
                // FadeInOut - Color (red, green. blue)
                Serial.println("     FadeInOut ");
                //FadeInOut(0xff, 0x00, 0x00); // red
                //FadeInOut(0xff, 0xff, 0xff); // white 
                FadeInOut(0x00, 0xff, 0x00); // blue
                Serial.println("     FadeInOut Exit");
                break;
              
              
    case 2  : 
//                Serial.println("     Strobe ");
//                // Strobe - Color (red, green, blue), number of flashes, flash speed, end pause
//                Strobe(0xff, 0xff, 0xff, 50, 50, 500);
//                Serial.println("     Strobe Exit");
//                break;
                Serial.println("      Fire");
                // Fire - Cooling rate, Sparking rate, speed delay
                Fire(80,120,2,1);
                Serial.println("     Fire Exit");
                break;
              

    case 3  : 
//                Serial.println("      HalloweenEyes");
//                // HalloweenEyes - Color (red, green, blue), Size of eye, space between eyes, fade (true/false), steps, fade delay, end pause
//                HalloweenEyes(0xff, 0x00, 0x00, 
//                              4, 8, 
//                              true, random(5,50), random(50,150), 
//                              random(1000, 3000));
//                HalloweenEyes(0xff, 0x00, 0x00, 
//                              4, 8, 
//                              true, random(5,50), random(50,150), 
//                              random(100, 3000));
//                Serial.println("     HalloweenEyes Exit");
//                break;
                Serial.println("     TwinkleRandom ");
                // TwinkleRandom - twinkle count, speed delay, only one (true/false)
                TwinkleRandom(125, 20, false);
                 Serial.println("     TwinkleRandom Exit");
               break;
              
              
    case 4  : 
                Serial.println("     CylonBounce ");
                // CylonBounce - Color (red, green, blue), eye size, speed delay, end pause
                CylonBounce(0xff, 0x00, 0x00, 15, 1, 20);
                 Serial.println("     CylonBounce Exit");
               break;
              
              
    case 5  : 
                //skipping this one for now
                // NewKITT - Color (red, green, blue), eye size, speed delay, end pause
                //NewKITT(0xff, 0x00, 0x00, 8, 10, 20);
//                Serial.println("      BouncingColoredBalls");
//                byte colors[3][3] = { {0xff, 0x00, 0x00},
//                                      {0xff, 0xff, 0xff},
//                                      {0x00, 0x00, 0xff} };
//                BouncingColoredBalls(3, colors, false);
//                Serial.println("     BouncingColoredBalls Exit");
                //BouncingColoredBalls(3);//int BallCount, byte colors[][3], boolean continuous) {
                Sparkle(random(255), random(255), random(255), 50);
                break;
              
              
    case 6  : 
                Serial.println("   Twinkle   ");
                // Twinkle - Color (red, green, blue), count, speed delay, only one twinkle (true/false) 
                Twinkle(0x00, 0x00, 0xff, 125, 20, false);
                Serial.println("     Twinkle Exit");
                break;
              
              
    case 7  :  
                Serial.println("     TwinkleRandom ");
                // TwinkleRandom - twinkle count, speed delay, only one (true/false)
                TwinkleRandom(125, 20, false);
                 Serial.println("     TwinkleRandom Exit");
               break;
              
              
    case 8  : 
                Serial.println("      Sparkle");
//                // Sparkle - Color (red, green, blue), speed delay
                Sparkle(255, 255, 255, 50);
                Serial.println("     Sparkle Exit");
                //theaterChase(0,0,0xff,200);
                break;
              
               
    case 9  : 
//                Serial.println("      SnowSparkle");
//                // SnowSparkle - Color (red, green, blue), sparkle delay, speed delay
//                SnowSparkle(0x10, 0x10, 0x10, 100, random(100,1000));
//                Serial.println("     SnowSparkle Exit");
//                break;
                Serial.println("      Fire");
                // Fire - Cooling rate, Sparking rate, speed delay
                Fire(80,120,2,2);
                Serial.println("     Fire Exit");
                break;
              
              
    case 10 : 
                Serial.println("     RunningLights ");
                // Running Lights - Color (red, green, blue), wave dealy
                RunningLights(0xff,0x00,0x00, 50);  // red
                //RunningLights(0xff,0xff,0xff, 10);  // white
                Serial.println("     RunningLights Exit");
                break;
              
              
    case 11 : 
                Serial.println("     colorWipe ");
                // colorWipe - Color (red, green, blue), speed delay
                colorWipe(0x00,0xff,0x00, 5);
                colorWipe(0x00,0x00,125, 5);
                Serial.println("     colorWipe Exit");
                break;
              

    case 12 : 
                Serial.println("      rainbowCycle");
                // rainbowCycle - speed delay
                rainbowCycle(5);
                Serial.println("     rainbowCycle Exit");
                break;
              

    case 13 : 
                Serial.println("      theaterChase");
                // theatherChase - Color (red, green, blue), speed delay
                theaterChase(0xff,0,0,100);
                Serial.println("     theaterChase Exit");
                break;
              

    case 14 : 
                Serial.println("     theaterChaseRainbow ");
                // theaterChaseRainbow - Speed delay
                theaterChaseRainbow(100);
                Serial.println("     theaterChaseRainbow Exit");
                break;
              

    case 15 : 
                Serial.println("      Fire");
                // Fire - Cooling rate, Sparking rate, speed delay
                Fire(80,120,2,3);
                Serial.println("     Fire Exit");
                break;
              


              
  }
  selectedEffect++;  ///take this out, just here to mimic button press
  if(selectedEffect>15)
    selectedEffect=0;
}

// *************************
// ** LEDEffect Functions **
// *************************
void RGBLoop(){
  for(int j = 0; j < 3; j++ ) { 
    // Fade IN
    for(int k = 0; k < 256; k++) { 
      switch(j) { 
        case 0: setAll(k,0,0); break;
        case 1: setAll(0,k,0); break;
        case 2: setAll(0,0,k); break;
      }
      showStrip();
      delay(1);
      //if there is a click we need to break out of playing
      if(exitPlaying()){ 
       setAll(0,0,0);
       return;
      }
    }
    // Fade OUT
    for(int k = 255; k >= 0; k--) { 
      switch(j) { 
        case 0: setAll(k,0,0); break;
        case 1: setAll(0,k,0); break;
        case 2: setAll(0,0,k); break;
      }
      showStrip();
      delay(1);
      //if there is a click we need to break out of playing
      if(exitPlaying()){ 
        setAll(0,0,0);
        return;
      }
    }
  }
}

void FadeInOut(byte red, byte green, byte blue){
  float r, g, b;
      
  for(int k = 0; k < 256; k=k+1) { 
    r = (k/256.0)*red;
    g = (k/256.0)*green;
    b = (k/256.0)*blue;
    setAll(r,g,b);
    showStrip();
    //if there is a click we need to break out of playing
    if(exitPlaying()){ 
      setAll(0,0,0);
      return;
    }
  }
     
  for(int k = 255; k >= 0; k=k-2) {
    r = (k/256.0)*red;
    g = (k/256.0)*green;
    b = (k/256.0)*blue;
    setAll(r,g,b);
    showStrip();
    //if there is a click we need to break out of playing
    if(exitPlaying()){ 
      setAll(0,0,0);
      return;
    }
  }
}

void Strobe(byte red, byte green, byte blue, int StrobeCount, int FlashDelay, int EndPause){
  for(int j = 0; j < StrobeCount; j++) {
    setAll(red,green,blue);
    showStrip();
    delay(FlashDelay);
    setAll(0,0,0);
    showStrip();
    delay(FlashDelay);
    //if there is a click we need to break out of playing
    if(exitPlaying()){ 
      setAll(0,0,0);
      return;
    }
  }
 
 delay(EndPause);
}

void HalloweenEyes(byte red, byte green, byte blue, 
                   int EyeWidth, int EyeSpace, 
                   boolean Fade, int Steps, int FadeDelay,
                   int EndPause){
  randomSeed(analogRead(0));
  
  int i;
  int StartPoint  = random( 0, LED_COUNT - (2*EyeWidth) - EyeSpace );
  int Start2ndEye = StartPoint + EyeWidth + EyeSpace;
  
  for(i = 0; i < EyeWidth; i++) {
    setPixel(StartPoint + i, red, green, blue);
    setPixel(Start2ndEye + i, red, green, blue);
  }
  
  showStrip();
  
  if(Fade==true) {
    float r, g, b;
  
    for(int j = Steps; j >= 0; j--) {
      r = j*(red/Steps);
      g = j*(green/Steps);
      b = j*(blue/Steps);
      
      for(i = 0; i < EyeWidth; i++) {
        setPixel(StartPoint + i, r, g, b);
        setPixel(Start2ndEye + i, r, g, b);
      }
      
      showStrip();
    //if there is a click we need to break out of playing
    if(exitPlaying()){ 
      setAll(0,0,0);
      return;
    }
      delay(FadeDelay);
    }
  }
  
  setAll(0,0,0); // Set all black
  
    //if there is a click we need to break out of playing
    if(exitPlaying()){ 
      setAll(0,0,0);
      return;
    }
  delay(EndPause);
}

void CylonBounce(byte red, byte green, byte blue, int EyeSize, int SpeedDelay, int ReturnDelay){

  for(int i = 0; i < LED_COUNT-EyeSize-2; i++) {
    setAll(0,0,0);
    setPixel(i, red/10, green/10, blue/10);
    for(int j = 1; j <= EyeSize; j++) {
      setPixel(i+j, red, green, blue); 
    }
    setPixel(i+EyeSize+1, red/10, green/10, blue/10);
    showStrip();
    delay(SpeedDelay);
    //if there is a click we need to break out of playing
    if(exitPlaying()){ 
      setAll(0,0,0);
      return;
    }
  }

  delay(ReturnDelay);

  for(int i = LED_COUNT-EyeSize-2; i > 0; i--) {
    setAll(0,0,0);
    setPixel(i, red/10, green/10, blue/10);
    for(int j = 1; j <= EyeSize; j++) {
      setPixel(i+j, red, green, blue); 
    }
    setPixel(i+EyeSize+1, red/10, green/10, blue/10);
    showStrip();
    delay(SpeedDelay);
    //if there is a click we need to break out of playing
    if(exitPlaying()){ 
      setAll(0,0,0);
      return;
    }
  }
  
  delay(ReturnDelay);
}

void NewKITT(byte red, byte green, byte blue, int EyeSize, int SpeedDelay, int ReturnDelay){
  RightToLeft(red, green, blue, EyeSize, SpeedDelay, ReturnDelay);
  //if there is a click we need to break out of playing
  if(exitPlaying()){ 
    setAll(0,0,0);
    return;
  }
  LeftToRight(red, green, blue, EyeSize, SpeedDelay, ReturnDelay);
  //if there is a click we need to break out of playing
  if(exitPlaying()){ 
    setAll(0,0,0);
    return;
  }
  OutsideToCenter(red, green, blue, EyeSize, SpeedDelay, ReturnDelay);
  //if there is a click we need to break out of playing
  if(exitPlaying()){ 
    setAll(0,0,0);
    return;
  }
  CenterToOutside(red, green, blue, EyeSize, SpeedDelay, ReturnDelay);
  //if there is a click we need to break out of playing
  if(exitPlaying()){ 
    setAll(0,0,0);
    return;
  }
  LeftToRight(red, green, blue, EyeSize, SpeedDelay, ReturnDelay);
  //if there is a click we need to break out of playing
  if(exitPlaying()){ 
    setAll(0,0,0);
    return;
  }
  RightToLeft(red, green, blue, EyeSize, SpeedDelay, ReturnDelay);
  //if there is a click we need to break out of playing
  if(exitPlaying()){ 
    setAll(0,0,0);
    return;
  }
  OutsideToCenter(red, green, blue, EyeSize, SpeedDelay, ReturnDelay);
  //if there is a click we need to break out of playing
  if(exitPlaying()){ 
    setAll(0,0,0);
    return;
  }
  CenterToOutside(red, green, blue, EyeSize, SpeedDelay, ReturnDelay);
}

// used by NewKITT
void CenterToOutside(byte red, byte green, byte blue, int EyeSize, int SpeedDelay, int ReturnDelay) {
  for(int i =((LED_COUNT-EyeSize)/2); i>=0; i--) {
    setAll(0,0,0);
    
    setPixel(i, red/10, green/10, blue/10);
    for(int j = 1; j <= EyeSize; j++) {
      setPixel(i+j, red, green, blue); 
    }
    setPixel(i+EyeSize+1, red/10, green/10, blue/10);
    
    setPixel(LED_COUNT-i, red/10, green/10, blue/10);
    for(int j = 1; j <= EyeSize; j++) {
      setPixel(LED_COUNT-i-j, red, green, blue); 
    }
    setPixel(LED_COUNT-i-EyeSize-1, red/10, green/10, blue/10);
    
    showStrip();
    delay(SpeedDelay);
    //if there is a click we need to break out of playing
    if(exitPlaying()){ 
      setAll(0,0,0);
      return;
    }
  }
  delay(ReturnDelay);
}

// used by NewKITT
void OutsideToCenter(byte red, byte green, byte blue, int EyeSize, int SpeedDelay, int ReturnDelay) {
  for(int i = 0; i<=((LED_COUNT-EyeSize)/2); i++) {
    setAll(0,0,0);
    
    setPixel(i, red/10, green/10, blue/10);
    for(int j = 1; j <= EyeSize; j++) {
      setPixel(i+j, red, green, blue); 
    }
    setPixel(i+EyeSize+1, red/10, green/10, blue/10);
    
    setPixel(LED_COUNT-i, red/10, green/10, blue/10);
    for(int j = 1; j <= EyeSize; j++) {
      setPixel(LED_COUNT-i-j, red, green, blue); 
    }
    setPixel(LED_COUNT-i-EyeSize-1, red/10, green/10, blue/10);
    
    showStrip();
    delay(SpeedDelay);
    //if there is a click we need to break out of playing
    if(exitPlaying()){ 
      setAll(0,0,0);
      return;
    }
  }
  delay(ReturnDelay);
}

// used by NewKITT
void LeftToRight(byte red, byte green, byte blue, int EyeSize, int SpeedDelay, int ReturnDelay) {
  for(int i = 0; i < LED_COUNT-EyeSize-2; i++) {
    setAll(0,0,0);
    setPixel(i, red/10, green/10, blue/10);
    for(int j = 1; j <= EyeSize; j++) {
      setPixel(i+j, red, green, blue); 
    }
    setPixel(i+EyeSize+1, red/10, green/10, blue/10);
    showStrip();
    delay(SpeedDelay);
    //if there is a click we need to break out of playing
    if(exitPlaying()){ 
      setAll(0,0,0);
      return;
    }
  }
  delay(ReturnDelay);
}

// used by NewKITT
void RightToLeft(byte red, byte green, byte blue, int EyeSize, int SpeedDelay, int ReturnDelay) {
  for(int i = LED_COUNT-EyeSize-2; i > 0; i--) {
    setAll(0,0,0);
    setPixel(i, red/10, green/10, blue/10);
    for(int j = 1; j <= EyeSize; j++) {
      setPixel(i+j, red, green, blue); 
    }
    //if there is a click we need to break out of playing
    if(exitPlaying()){ 
      setAll(0,0,0);
      return;
    }
    setPixel(i+EyeSize+1, red/10, green/10, blue/10);
    showStrip();
    delay(SpeedDelay);
  }
  delay(ReturnDelay);
}

void Twinkle(byte red, byte green, byte blue, int Count, int SpeedDelay, boolean OnlyOne) {
  setAll(0,0,0);
  
  for (int i=0; i<Count; i++) {
     setPixel(random(LED_COUNT),red,green,blue);
     showStrip();
     delay(SpeedDelay);
     if(OnlyOne) { 
       setAll(0,0,0); 
     }
    //if there is a click we need to break out of playing
    if(exitPlaying()){ 
      setAll(0,0,0);
      return;
    }
   }
  
  delay(SpeedDelay);
}

void TwinkleRandom(int Count, int SpeedDelay, boolean OnlyOne) {
  setAll(0,0,0);
  
  for (int i=0; i<Count; i++) {
     setPixel(random(LED_COUNT),random(0,255),random(0,255),random(0,255));
     showStrip();
     delay(SpeedDelay);
     if(OnlyOne) { 
       setAll(0,0,0); 
     }
    //if there is a click we need to break out of playing
    if(exitPlaying()){ 
      setAll(0,0,0);
      return;
    }
   }
  
  delay(SpeedDelay);
}

void Sparkle(byte red, byte green, byte blue, int SpeedDelay) {
  while(1){
      int Pixel = random(LED_COUNT);
      setPixel(Pixel,red,green,blue);
      showStrip();
      delay(SpeedDelay);
      setPixel(Pixel,0,0,0);
      //if there is a click we need to break out of playing
      if(exitPlaying()){ 
        setAll(0,0,0);
        return;
      }
  }
}

void SnowSparkle(byte red, byte green, byte blue, int SparkleDelay, int SpeedDelay) {
  setAll(red,green,blue);
  
  int Pixel = random(LED_COUNT);
  setPixel(Pixel,0xff,0xff,0xff);
  showStrip();
  delay(SparkleDelay);
  setPixel(Pixel,red,green,blue);
  showStrip();
  delay(SpeedDelay);
  //if there is a click we need to break out of playing
  if(exitPlaying()){ 
    setAll(0,0,0);
    return;
  }
}

void RunningLights(byte red, byte green, byte blue, int WaveDelay) {
  int Position=0;
  
  for(int i=0; i<LED_COUNT*2; i++)
  {
      Position++; // = 0; //Position + Rate;
      for(int i=0; i<LED_COUNT; i++) {
        // sine wave, 3 offset waves make a rainbow!
        //float level = sin(i+Position) * 127 + 128;
        //setPixel(i,level,0,0);
        //float level = sin(i+Position) * 127 + 128;
        setPixel(i,((sin(i+Position) * 127 + 128)/255)*red,
                   ((sin(i+Position) * 127 + 128)/255)*green,
                   ((sin(i+Position) * 127 + 128)/255)*blue);
      }
      
      showStrip();
      delay(WaveDelay);
      //if there is a click we need to break out of playing
      if(exitPlaying()){ 
        setAll(0,0,0);
        return;
      }
  }
}

void colorWipe(byte red, byte green, byte blue, int SpeedDelay) {
  for(uint16_t i=0; i<LED_COUNT; i++) {
      setPixel(i, red, green, blue);
      showStrip();
      delay(SpeedDelay);
      //if there is a click we need to break out of playing
      if(exitPlaying()){ 
        setAll(0,0,0);
        return;
      }
  }
}

void rainbowCycle(int SpeedDelay) {
  byte *c;
  uint16_t i, j;

  for(j=0; j<256*5; j++) { // 5 cycles of all colors on wheel
    for(i=0; i< LED_COUNT; i++) {
      c=Wheel(((i * 256 / LED_COUNT) + j) & 255);
      setPixel(i, *c, *(c+1), *(c+2));
    }
    showStrip();
    //if there is a click we need to break out of playing
    if(exitPlaying()){ 
      setAll(0,0,0);
      return;
    }

    delay(SpeedDelay);
  }
}

// used by rainbowCycle and theaterChaseRainbow
byte * Wheel(byte WheelPos) {
  static byte c[3];
  
  if(WheelPos < 85) {
   c[0]=WheelPos * 3;
   c[1]=255 - WheelPos * 3;
   c[2]=0;
  } else if(WheelPos < 170) {
   WheelPos -= 85;
   c[0]=255 - WheelPos * 3;
   c[1]=0;
   c[2]=WheelPos * 3;
  } else {
   WheelPos -= 170;
   c[0]=0;
   c[1]=WheelPos * 3;
   c[2]=255 - WheelPos * 3;
  }

  return c;
}

void theaterChase(byte red, byte green, byte blue, int SpeedDelay) {
  for (int j=0; j<10; j++) {  //do 30 cycles of chasing
    for (int q=0; q < 3; q++) {
      for (int i=0; i < LED_COUNT; i=i+3) {
        setPixel(i+q, red, green, blue);    //turn every third pixel on
      }
      //if there is a click we need to break out of playing
      if(exitPlaying()){ 
        setAll(0,0,0);
        return;
      }
      showStrip();
     
      delay(SpeedDelay);
     
      for (int i=0; i < LED_COUNT; i=i+3) {
        setPixel(i+q, 0,0,0);        //turn every third pixel off
      }
    }
  }
}

void theaterChaseRainbow(int SpeedDelay) {
  byte *c;
  
  for (int j=0; j < 256; j++) {     // cycle all 256 colors in the wheel
    for (int q=0; q < 3; q++) {
        for (int i=0; i < LED_COUNT; i=i+3) {
          c = Wheel( (i+j) % 255);
          setPixel(i+q, *c, *(c+1), *(c+2));    //turn every third pixel on
        }
        //if there is a click we need to break out of playing
        if(exitPlaying()){ 
          setAll(0,0,0);
          return;
        }

        
        showStrip();
       
        delay(SpeedDelay);
       
        for (int i=0; i < LED_COUNT; i=i+3) {
          setPixel(i+q, 0,0,0);        //turn every third pixel off
        }
    }
  }
}

void Fire(int Cooling, int Sparking, int SpeedDelay, int color) {
  static byte heat[LED_COUNT/2];
  int cooldown;
  unsigned long timeMillis = millis();
  setAll(0,0,0);
  showStrip();
  while(millis()-timeMillis < 30000){
    // Step 1.  Cool down every cell a little
    for( int i = 0; i < LED_COUNT/2; i++) {
      cooldown = random(0, ((Cooling * 10) / LED_COUNT) + 2);
      
      if(cooldown>heat[i]) {
        heat[i]=0;
      } else {
        heat[i]=heat[i]-cooldown;
      }
    }
    
    // Step 2.  Heat from each cell drifts 'up' and diffuses a little
    for( int k= LED_COUNT/2 - 1; k >= 2; k--) {
      heat[k] = (heat[k - 1] + heat[k - 2] + heat[k - 2]) / 3;
    }
      
    // Step 3.  Randomly ignite new 'sparks' near the bottom
    if( random(255) < Sparking ) {
      int y = random(7);
      heat[y] = heat[y] + random(160,255);
      //heat[y] = random(160,255);
    }
  
    // Step 4.  Convert heat to LED colors
    for( int j = 0; j < LED_COUNT/2; j++) {
      setPixelHeatColor(j, heat[j] , color);
    }
  
    //if there is a click we need to break out of playing
    if(exitPlaying()){ 
      setAll(0,0,0);
      return;
    }
  
    showStrip();
    delay(SpeedDelay);
  }
  setAll(0,0,0);
  showStrip();
}

void setPixelHeatColor (int Pixel, byte temperature, int color) {
  // Scale 'heat' down from 0-255 to 0-191
  byte t192 = round((temperature/255.0)*191);
 
  // calculate ramp up from
  byte heatramp = t192 & 0x3F; // 0..63
  heatramp <<= 2; // scale up to 0..252
 
  // figure out which third of the spectrum we're in:
  if( t192 > 0x80) {                     // hottest
    //greenish
    if(color==1){
      setPixel(Pixel, 255, 255, heatramp);
      setPixel(LED_COUNT-Pixel, 255, 255, heatramp);
    } else if(color==2){
      setPixel(Pixel, heatramp, 255, 255);
      setPixel(LED_COUNT-Pixel, heatramp, 255, 255);
    } else if (color==3){
      setPixel(Pixel, 255, heatramp,  255);
      setPixel(LED_COUNT-Pixel, 255, heatramp, 255);
      
    }else if (color==4){
      setPixel(Pixel, 255, heatramp,  255);
      setPixel(LED_COUNT-Pixel, 255, heatramp, 255);
      
    }
    
  } else if( t192 > 0x40 ) {             // middle
    if(color==1){
      setPixel(Pixel, 255, heatramp, 0);
      setPixel(LED_COUNT-Pixel, 255, heatramp, 0);
    } else if(color==2){
    //blueish
      setPixel(Pixel, 0,  heatramp, 255);
      setPixel(LED_COUNT-Pixel, 0,  heatramp, 255);
    } else if(color==3){
      setPixel(Pixel, heatramp, 255, 0);
      setPixel(LED_COUNT-Pixel, heatramp, 255, 0);
    }else if(color==4){
      setPixel(Pixel, heatramp, 255, heatramp);
      setPixel(LED_COUNT-Pixel, heatramp, 255, heatramp);
    }
  } else {                               // coolest
    if(color==1){
      setPixel(Pixel, heatramp, 0, 0);
      setPixel(LED_COUNT-Pixel, heatramp, 0, 0);
    }else if(color==2){
      setPixel(Pixel, 0,0,heatramp);
      setPixel(LED_COUNT-Pixel, 0,0,heatramp);
    } else if (color==3){
      setPixel(Pixel, 0,heatramp,0);
      setPixel(LED_COUNT-Pixel, 0,heatramp,0);
    }else if (color==4){
      setPixel(Pixel, heatramp,heatramp,0);
      setPixel(LED_COUNT-Pixel, heatramp,heatramp,0);
    }
  }
}

void BouncingColoredBalls(int BallCount, byte colors[][3], boolean continuous) {
  float Gravity = -9.81;
  int StartHeight = 1;
  
  float Height[BallCount];
  float ImpactVelocityStart = sqrt( -2 * Gravity * StartHeight );
  float ImpactVelocity[BallCount];
  float TimeSinceLastBounce[BallCount];
  int   Position[BallCount];
  long  ClockTimeSinceLastBounce[BallCount];
  float Dampening[BallCount];
  boolean ballBouncing[BallCount];
  boolean ballsStillBouncing = true;
  
  for (int i = 0 ; i < BallCount ; i++) {   
    ClockTimeSinceLastBounce[i] = millis();
    Height[i] = StartHeight;
    Position[i] = 0; 
    ImpactVelocity[i] = ImpactVelocityStart;
    TimeSinceLastBounce[i] = 0;
    Dampening[i] = 0.90 - float(i)/pow(BallCount,2);
    ballBouncing[i]=true; 
  }

  while (ballsStillBouncing) {
    for (int i = 0 ; i < BallCount ; i++) {
      TimeSinceLastBounce[i] =  millis() - ClockTimeSinceLastBounce[i];
      Height[i] = 0.5 * Gravity * pow( TimeSinceLastBounce[i]/1000 , 2.0 ) + ImpactVelocity[i] * TimeSinceLastBounce[i]/1000;
  
      if ( Height[i] < 0 ) {                      
        Height[i] = 0;
        ImpactVelocity[i] = Dampening[i] * ImpactVelocity[i];
        ClockTimeSinceLastBounce[i] = millis();
  
        if ( ImpactVelocity[i] < 0.01 ) {
          if (continuous) {
            ImpactVelocity[i] = ImpactVelocityStart;
          } else {
            ballBouncing[i]=false;
          }
        }
      }
      Position[i] = round( Height[i] * (LED_COUNT/2 - 1) / StartHeight);
    }

    ballsStillBouncing = false; // assume no balls bouncing
    for (int i = 0 ; i < BallCount ; i++) {
      setPixel(Position[i],colors[i][0],colors[i][1],colors[i][2]);
      if ( ballBouncing[i] ) {
        ballsStillBouncing = true;
      }
    }

    //if there is a click we need to break out of playing
    if(exitPlaying()){ 
      setAll(0,0,0);
      return;
    }

    showStrip();
    setAll(0,0,0);
  }
}

void meteorRain(byte red, byte green, byte blue, byte meteorSize, byte meteorTrailDecay, boolean meteorRandomDecay, int SpeedDelay) {  
  setAll(0,0,0);
  
//  for(int i = 0; i < LED_COUNT+LED_COUNT; i++) {
  for(int i = 0; i < LED_COUNT/2+20; i=i+2) {
    //something strange started to happen when this loop got near the end so we now just look for 122 and exit
    //it started after updating to the latest FastLED lib
    if(i==122){ 
      setAll(0,0,0);
      return;
    }
    // fade brightness all LEDs one step
    //for(int j=0; j<LED_COUNT; j++) {
    for(int j=0; j<i; j++) {
      if( (!meteorRandomDecay) || (random(10)>5) ) {
        fadeToBlack((LED_COUNT/2)-j, meteorTrailDecay );        
        fadeToBlack((LED_COUNT/2)+j, meteorTrailDecay );        
      }
    }
    
    // draw meteor
    for(int j = 0; j < meteorSize; j++) {
      if( ( i-j <LED_COUNT/2) && (i-j>=0) ) {
        setPixel((LED_COUNT/2)+(i-j), red, green, blue);
        setPixel((LED_COUNT/2)-(i-j), red, green, blue);
      } 
    }
   
    showStrip();
    //delay(SpeedDelay);

    //if there is a click we need to break out of playing
    if(exitPlaying()){ 
      setAll(0,0,0);
      return;
    }
  }

}

// used by meteorrain
void fadeToBlack(int ledNo, byte fadeValue) {
 #ifdef ADAFRUIT_NEOPIXEL_H 
    // NeoPixel
    uint32_t oldColor;
    uint8_t r, g, b;
    int value;
    
    oldColor = strip.getPixelColor(ledNo);
    r = (oldColor & 0x00ff0000UL) >> 16;
    g = (oldColor & 0x0000ff00UL) >> 8;
    b = (oldColor & 0x000000ffUL);

    r=(r<=10)? 0 : (int) r-(r*fadeValue/256);
    g=(g<=10)? 0 : (int) g-(g*fadeValue/256);
    b=(b<=10)? 0 : (int) b-(b*fadeValue/256);
    
    strip.setPixelColor(ledNo, r,g,b);
 #endif
 #ifndef ADAFRUIT_NEOPIXEL_H
   // FastLED
   leds[ledNo].fadeToBlackBy( fadeValue );
 #endif  
}

// *** REPLACE TO HERE ***



// ***************************************
// ** FastLed/NeoPixel Common Functions **
// ***************************************

// Apply LED color changes
void showStrip() {
 #ifdef ADAFRUIT_NEOPIXEL_H 
   // NeoPixel
     Serial.println("NEOPIXEL lib");

   strip.show();
 #endif
 #ifndef ADAFRUIT_NEOPIXEL_H
   // FastLED
   FastLED.show();
 #endif
}


//basically every where I use setPixel() I call it 2 at a time.  This is because my LED strip is
//folded over, so LED  #1 is on the other side of LED # 'LED_COUNT'

// Set a LED color (not yet visible)
void setPixel(int Pixel, byte red, byte green, byte blue) {
 #ifdef ADAFRUIT_NEOPIXEL_H 
   // NeoPixel
   strip.setPixelColor(Pixel, strip.Color(red, green, blue));
 #endif
 #ifndef ADAFRUIT_NEOPIXEL_H 
   // FastLED
   leds[Pixel].r = red;
   leds[Pixel].g = green;
   leds[Pixel].b = blue;
 #endif
}

// Set all LEDs to a given color and apply it (visible)
void setAll(byte red, byte green, byte blue) {
  for(int i = 0; i < LED_COUNT; i++ ) {
    setPixel(i, red, green, blue); 
  }
  showStrip();
}
